/*
 * Schnupperstudium 2019
 *
 * Tiny Stylophone
 *
 * 3.2.2019 S. Tschirley
 *
 * main:
 * +-----------------------------+             +---------------------------+
 * | Initialisierung von allem   |             |  Timer ISR                |
 * +----------------+------------+             +---------------------------+
 *                  |                          |                           |
 * +---------------->                          |  Notenwert in             |
 * |                |                          |  Zaehlregister eintragen  |
 * |     +----------v------------+             |                           |
 * |     | Ports einlesen        |             +---------------------------+
 * |     +----------+------------+
 * |                |
 * |     +----------v------------+
 * |     | gedruecktes Pad finden|
 * |     +----------+------------+
 * |                |
 * |     +----------v------------+    nein
 * |     | Ist die Pad  0x00 ?   +---------------+
 * |     +----------+------------+               |
 * |                |                            |
 * |                | ja                         |
 * |                |                            |
 * |     +----------v------------+   +-----------v-----------+
 * |     | Kein Sound            |   | Note bestimmen        |
 * |     +----------+------------+   +-----------+-----------+
 * |                |                            |
 * |                |                +-----------v-----------+
 * |                |                | Sound einschalten     |
 * |                |                +-----------+-----------+
 * |                |                            |
 * +----------------v----------------------------+
 *
 */

#include <msp430g2553.h>
#include "music.h"

/*
 * Lautsprecher
 */
#define SPEAKER BIT5
#define SPEAKER_DIR P2DIR
#define SPEAKER_OUT P2OUT

/*
 * Bedienfelder an Port P1.0 bi P1.7 und P2.0 bis P2.3
 */
#define PAD0	BIT0 // C
#define PAD1	BIT1 // C#
#define PAD2    BIT2 // D
#define PAD3	BIT3 // D#
#define PAD4	BIT4 // E
#define PAD5	BIT5 // F
#define PAD6	BIT6 // F#
#define PAD7	BIT7 // G
#define PAD_P1_BITS ( PAD0|PAD1|PAD2|PAD3|PAD4|PAD5|PAD6|PAD7 )
#define PAD_P1_DIR P1DIR
#define PAD_P1_IN  P1IN
#define PAD8	BIT0 // G#
#define PAD9	BIT1 // A
#define PAD10   BIT2 // A#
#define PAD11	BIT3 // H
#define PAD_P2_BITS ( PAD8|PAD9|PAD10|PAD11 )
#define PAD_P2_DIR P2DIR
#define PAD_P2_IN  P2IN


/* Schalter fuer Sound                                   */
volatile unsigned int note_on = FALSE;
/* Speicher fuer die aktuelle Note, die gespielt wird    */
volatile unsigned int current_note;
/* Variable fuer PAD-Wert                                */
volatile unsigned int pad_value;
/* fuer Pausen                                           */
volatile unsigned int ms_elapsed = 0;
/* Puffervariablen fuer das Einlesen der beiden Ports    */
unsigned char p1_val = 0;
unsigned char p2_val = 0;

/****************************************************/
/*
 * Funktionen fuer den Programmablauf
 */

/*
 * Initialisierung des Controllers
 * Hier werden alle Peripheriebaugruppen voreingestellt
 * so dass Pads und Lautsprecher angesteuert
 * werden koennen
 */
void ConfigController( void )
{
    /* Watchdog ausschalten                         */
    WDTCTL = WDTPW | WDTHOLD;

    /* Taktfrequenz auf 1 MHz einstellen            */
    DCOCTL  = CALDCO_1MHZ;
    BCSCTL1 = CALBC1_1MHZ;

    /* Wenn interne Pull-Ups verwendet werden, dann */
    /* werden die konfiguriert                      */
    P1OUT = PAD_P1_BITS;
    P1REN = PAD_P1_BITS;
    P2OUT = PAD_P2_BITS;
    P2REN = PAD_P2_BITS;
    /* Pads sind Eingaenge                          */
    PAD_P1_DIR &= ~PAD_P1_BITS;
    PAD_P1_DIR &= ~PAD_P1_DIR;
    /* Pads sind Eingaenge                          */
    PAD_P2_DIR &= ~PAD_P2_BITS;
    PAD_P2_DIR &= ~PAD_P2_DIR;

    /* Lautsprecheranschluss ist Ausgang            */
    SPEAKER_DIR |= SPEAKER;

    /* Einstellen des Timers                        */
    TACTL |= TASSEL_2 | ID_0 | MC_2 | TAIE;
    TACCTL0 |= CCIE;
    TACCTL1 |= CCIE;

    /* Alle interrupts werden global eingeschaltet. */
    __bis_SR_register(GIE);
}

/****************************************************/
/*
 * Interrupt Service Routinen (ISR)
 */

/*
 * TIMER A0 ISR
 *
 * Dieser Timer macht die Musik
 */
#pragma vector=TIMER0_A0_VECTOR
__interrupt void Timer0_A0 (void)
{
    /* Timervergleichsregister setzen               */
    TACCR0 += current_note;

    /* Wenn Pad gespielt wird,                      */
    /* dann Lautsprecherpin toggeln.                */
    if (note_on) {
       SPEAKER_OUT ^= SPEAKER;
    }
}

/*
 * TIMER A1 ISR
 */
#pragma vector=TIMER0_A1_VECTOR
__interrupt void Timer0_A1 (void)
{
    switch (TAIV)
      {
        case 2: /* CCR1 */
          TACCR1 += 1000; /* 1 ms */
          ms_elapsed++;
          break;
      }
}

/*
 * nicht verwentete Interrupts werden als leere Funktionen
 * implementiert, damit nicht irgendwas eingefangen wird.
 * Das sollte nich passieren, da die alle ausgeschaltet sind,
 * aber man kann ja nie wissen...
 */
#pragma vector=ADC10_VECTOR
__interrupt void ADC10_ISR( void ){}

#pragma vector=PORT2_VECTOR
__interrupt void Port_2_ISR( void ){}

#pragma vector=NMI_VECTOR
__interrupt void NMI_ISR( void ){}

#pragma vector=WDT_VECTOR
__interrupt void WDT_ISR( void ){}


/****************************************************/
/*
 * Hauptprogramm
 */

/*
 * Das ist das Hauptprogramm.
 * Was da genau passiert, steht in den Kommentaren im Quelltext.
 */
int main(void) {
    /*  Initialisierung des Controllers                           */
	ConfigController();
	
    /* Es startet eine Endlosschleife, die der Controller         */
    /* abarbeitet, bis ihm der Strom ausgeht.                     */
	while(1)
	{
		/* Die Pads werden eingelesen und invertiert              */
		p1_val = ~PAD_P1_IN;
		p2_val = ~PAD_P2_IN & PAD_P2_BITS;
		/* Was da gelesen wurde kommt in eine einzelne Variable   */
		pad_value = p2_val<<8;
		pad_value |= p1_val;
		/* Wenn nichts gespielt wird, dann wird kein Ton erzeugt  */
		/* wenn doch, dann wird bestimmt, welcher Ton gespielt    */
		/* werden soll. Der wird dann in die Variable             */
		/*   current_note  hineingeschrieben                      */
		if ( pad_value == 0 )
		{
			note_on = FALSE;
		}
		else
		{
			switch( pad_value )
			{
			case 0x0001:
				current_note = C4;
				break;
			case 0x0002:
				current_note = Cs4;
				break;
			case 0x0008:
				current_note = D4;
				break;
			case 0x0004:
				current_note = Ds4;
				break;
			case 0x0010:
				current_note = E4;
				break;
			case 0x0020:
				current_note = F4;
				break;
			case 0x0040:
				current_note = Fs4;
				break;
			case 0x0080:
				current_note = G4;
				break;
			case 0x0100:
				current_note = Gs4;
				break;
			case 0x0200:
				current_note = A4;
				break;
			case 0x0400:
				current_note = As4;
				break;
			case 0x0800:
				current_note = B4;
				break;
			default:
				break;
			}
			/* Nun wird noch signalisiert, dass ein Ton erklingen */
			/* soll, das macht dann der Timer                     */
			note_on = TRUE;
		}
	}
	return 0;
}
